package com.sun.java.util.jar.pack;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;

class AdaptiveCoding implements CodingMethod
{
    CodingMethod headCoding;
    int headLength;
    CodingMethod tailCoding;
    public static final int KX_MIN = 0;
    public static final int KX_MAX = 3;
    public static final int KX_LG2BASE = 4;
    public static final int KX_BASE = 16;
    public static final int KB_MIN = 0;
    public static final int KB_MAX = 255;
    public static final int KB_OFFSET = 1;
    public static final int KB_DEFAULT = 3;
    
    public AdaptiveCoding(int headLength, CodingMethod headCoding, CodingMethod tailCoding) {
        assert !(!isCodableLength(headLength));
        this.headLength = headLength;
        this.headCoding = headCoding;
        this.tailCoding = tailCoding;
        return;
    }
    
    public void setHeadCoding(CodingMethod headCoding) {
        this.headCoding = headCoding;
    }
    
    public void setHeadLength(int headLength) {
        assert !(!isCodableLength(headLength));
        this.headLength = headLength;
        return;
    }
    
    public void setTailCoding(CodingMethod tailCoding) {
        this.tailCoding = tailCoding;
    }
    
    public boolean isTrivial() {
        return this.headCoding == this.tailCoding;
    }
    
    @Override
    public void writeArrayTo(OutputStream outputStream, int[] array, int n, int n2) throws IOException {
        writeArray(this, outputStream, array, n, n2);
    }
    
    private static void writeArray(AdaptiveCoding adaptiveCoding, OutputStream outputStream, int[] array, int n, int n2) throws IOException {
        while (true) {
            int n3 = n + adaptiveCoding.headLength;
            assert n3 <= n2;
            adaptiveCoding.headCoding.writeArrayTo(outputStream, array, n, n3);
            n = n3;
            if ((adaptiveCoding.tailCoding instanceof AdaptiveCoding)) {
                adaptiveCoding = (AdaptiveCoding)adaptiveCoding.tailCoding;
            }
            else {
                adaptiveCoding.tailCoding.writeArrayTo(outputStream, array, n, n2);
                return;
            }
        }
    }
    
    @Override
    public void readArrayFrom(InputStream inputStream, int[] array, int n, int n2) throws IOException {
        readArray(this, inputStream, array, n, n2);
    }
    
    private static void readArray(AdaptiveCoding adaptiveCoding, InputStream inputStream, int[] array, int n, int n2) throws IOException {
        while (true) {
            int n3 = n + adaptiveCoding.headLength;
            assert n3 <= n2;
            adaptiveCoding.headCoding.readArrayFrom(inputStream, array, n, n3);
            n = n3;
            if ((adaptiveCoding.tailCoding instanceof AdaptiveCoding)) {
                adaptiveCoding = (AdaptiveCoding)adaptiveCoding.tailCoding;
            }
            else {
                adaptiveCoding.tailCoding.readArrayFrom(inputStream, array, n, n2);
                return;
            }
        }
    }
    
    static int getKXOf(int n) {
        int i = 0;
        while (i <= 3) {
            if ((n - 1 & 0xFFFFFF00) == 0x0) {
                return i;
            }
            n >>>= 4;
            ++i;
        }
        return -1;
    }
    
    static int getKBOf(int n) {
        int kxOf = getKXOf(n);
        if (kxOf < 0) {
            return -1;
        }
        n >>>= kxOf * 4;
        return n - 1;
    }
    
    static int decodeK(int n, int n2) {
        assert 0 <= n && n <= 3;
        assert 0 <= n2 && n2 <= 255;
        return n2 + 1 << n * 4;
    }
    
    static int getNextK(int n) {
        if (n <= 0) {
            return 1;
        }
        int kxOf = getKXOf(n);
        if (kxOf < 0) {
            return Integer.MAX_VALUE;
        }
        int n2 = 1 << kxOf * 4;
        int n3 = 255 << kxOf * 4;
        int n4 = n + n2 & ~(n2 - 1);
        if ((n4 - n2 & ~n3) == 0x0) {
            assert getKXOf(n4) == kxOf;
            return n4;
        }
        else {
            if (kxOf == 3) {
                return Integer.MAX_VALUE;
            }
            ++kxOf;
            int n5 = (n4 | (n3 & ~(255 << kxOf * 4))) + n2;
            assert getKXOf(n5) == kxOf;
            return n5;
        }
    }
    
    public static boolean isCodableLength(int n) {
        int kxOf = getKXOf(n);
        return kxOf >= 0 && (n - (1 << kxOf * 4) & ~(255 << kxOf * 4)) == 0x0;
    }
    
    @Override
    public byte[] getMetaCoding(Coding coding) {
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream(10);
        try {
            makeMetaCoding(this, coding, byteArrayOutputStream);
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
        return byteArrayOutputStream.toByteArray();
    }
    
    private static void makeMetaCoding(AdaptiveCoding adaptiveCoding, Coding coding, ByteArrayOutputStream byteArrayOutputStream) throws IOException {
        while (true) {
            CodingMethod headCoding = adaptiveCoding.headCoding;
            int headLength = adaptiveCoding.headLength;
            CodingMethod tailCoding = adaptiveCoding.tailCoding;
            assert !(!isCodableLength(headLength));
            int n = (headCoding == coding) ? 1 : 0;
            int n2 = (tailCoding == coding) ? 1 : 0;
            if (n + n2 > 1) {
                n2 = 0;
            }
            int n3 = 1 * n + 2 * n2;
            assert n3 < 3;
            int kxOf = getKXOf(headLength);
            int kbOf = getKBOf(headLength);
            assert decodeK(kxOf, kbOf) == headLength;
            int n4 = (kbOf != 3) ? 1 : 0;
            byteArrayOutputStream.write(117 + kxOf + 4 * n4 + 8 * n3);
            if (n4 != 0) {
                byteArrayOutputStream.write(kbOf);
            }
            if (n == 0) {
                byteArrayOutputStream.write(headCoding.getMetaCoding(coding));
            }
            if ((tailCoding instanceof AdaptiveCoding)) {
                adaptiveCoding = (AdaptiveCoding)tailCoding;
            }
            else {
                if (n2 == 0) {
                    byteArrayOutputStream.write(tailCoding.getMetaCoding(coding));
                }
                return;
            }
        }
    }
    
    public static int parseMetaCoding(byte[] array, int n, Coding coding, CodingMethod[] array2) {
        int n2 = array[n++] & 0xFF;
        if (n2 < 117 || n2 >= 141) {
            return n - 1;
        }
        Object o = null;
        int i = 1;
        while (i != 0) {
            i = 0;
            assert n2 >= 117;
            n2 -= 117;
            int n3 = n2 % 4;
            int n4 = n2 / 4 % 2;
            int n5 = n2 / 8;
            assert n5 < 3;
            int n6 = n5 & 0x1;
            int n7 = n5 & 0x2;
            CodingMethod[] array3 = { coding };
            CodingMethod[] array4 = { coding };
            int n8 = 3;
            if (n4 != 0) {
                n8 = (array[n++] & 0xFF);
            }
            if (n6 == 0) {
                n = BandStructure.parseMetaCoding(array, n, coding, array3);
            }
            if (n7 == 0 && (n2 = (array[n] & 0xFF)) >= 117 && n2 < 141) {
                ++n;
                i = 1;
            }
            else if (n7 == 0) {
                n = BandStructure.parseMetaCoding(array, n, coding, array4);
            }
            AdaptiveCoding tailCoding = new AdaptiveCoding(decodeK(n3, n8), array3[0], array4[0]);
            if (o == null) {
                array2[0] = tailCoding;
            }
            else {
                o.tailCoding = tailCoding;
            }
            o = tailCoding;
        }
        return n;
    }
    
    private String keyString(CodingMethod codingMethod) {
        if ((codingMethod instanceof Coding)) {
            return ((Coding)codingMethod).keyString();
        }
        return codingMethod.toString();
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(20);
        sb.append("run(");
        while (true) {
            sb.append(this.headLength).append("*");
            sb.append(this.keyString(this.headCoding));
            if ((this.tailCoding instanceof AdaptiveCoding)) {
                this = (AdaptiveCoding)this.tailCoding;
                sb.append(" ");
            }
            else {
                break;
            }
        }
        sb.append(" **").append(this.keyString(this.tailCoding));
        sb.append(")");
        return sb.toString();
    }
}
